/*
 * mainloop.hpp
 *
 *  Created on: 22.11.2017
 *      Author: ubuntu
 */

#ifndef SRC_MODULES_MAIN_LOOP_MAINLOOP_HPP_
#define SRC_MODULES_MAIN_LOOP_MAINLOOP_HPP_


#include "../../OpenCV_includes.hpp"


#include "../helper_functions/helper_functions.hpp"
#include "../Function/Function.hpp"
#include "../concatenation/concatenation.hpp"
#include "../edge_detection/edge_detection.hpp"
#include "../framekeeper/framekeeper.hpp"
#include "../coordinate_transform/coordinate_transform.hpp"
#include "../trajectory_planner/ego_lane/EgoLane.hpp"


#include "../coordinate_system/coordinatesystem.hpp"



class main_loop {

	private:

		cv::Scalar color_l;
		cv::Scalar color_r;
		cv::Scalar color_m;


		cv::Mat src_processed;
		cv::Mat grad_x, grad_y, detected_edges;
		cv::Mat si_units;
		cv::Mat test;
		cv::Mat coordinates;

		std::vector<cv::Point> contour_l;
		std::vector<cv::Point> contour_r;
		std::vector<cv::Point> contour_m;

		std::vector<Function::pPoint> pcontours_l;
		std::vector<Function::pPoint> pcontours_r;
		std::vector<Function::pPoint> pcontours_m;

		std::vector<Function::pPoint> real_l;
		std::vector<Function::pPoint> real_r;
		std::vector<Function::pPoint> real_m;

		Function::vec_func vec_l;
		Function::vec_func vec_r;
		Function::vec_func vec_m;

		std::vector<Function::pPoint> fitted_l;
		std::vector<Function::pPoint> fitted_r;
		std::vector<Function::pPoint> fitted_m;


		std::vector<Function::pPoint> lane_pPoints;
		std::vector<Function::pPoint> lane_pPoints_t;
		std::vector<cv::Point> lane_Points;

		// ------- get the parameters --------
		// cam parameter

		float camera_height; // in cm
		float camera_angle; // in degrees and float
		float aperture_angle_x; // in degrees
		float aperture_angle_y; // in degrees
		// both in cm
		Function::pPoint camera_translation;

		// edge detection parameter
		unsigned short gradient_threshold; // min threshold for the
										   // gradients on the edge
										   // (should be around 900-1200

		int line_distance_threshold;
		int line_spread;
		int min_line_length;

		// Get video information
		int width;
		int height;


		// calculate the translation of the roi
		int roi_translation_x; // in float ratio should be around 0.125
		int roi_translation_y; // in float ratio should be around 0.5

		int pow_poly; // the max power of the polynomial fitting function TODO einbinden

		int point_amount;

		// initialize the modules based upon the given parameters and variables
		EdgeDetection edge;
		Concatenation c;
		CoordinateTransform coordinate;
		Coordinate_System coord;

		Ego_Lane ego_lane;

		FrameKeeper frame;
		
		bool screen_exists;


	public:

		main_loop(char** argv) : color_l(0, 170, 255),
								 color_r(255, 150, 70),
								 color_m(150, 255, 150),
								 camera_height(std::stof(argv[1])),
								 camera_angle(std::stof(argv[2])),
								 aperture_angle_x(std::stof(argv[3])),
								 aperture_angle_y(std::stof(argv[4])),
								 camera_translation(std::stof(argv[5]), std::stof(argv[6])),
								 gradient_threshold(std::stoi(argv[7])),
								 line_distance_threshold(std::stoi(argv[8])),
								 line_spread(std::stoi(argv[9])),
								 min_line_length(std::stoi(argv[10])),
								 width(std::stoi(argv[11])),
								 height(std::stoi(argv[12])),
								 roi_translation_x(width * std::stof(argv[13])),
								 roi_translation_y(height * std::stof(argv[14])),
								 pow_poly(std::stoi(argv[15])),
								 point_amount(std::stoi(argv[16])),
								 edge(height, width,
										 roi_translation_x,
										 roi_translation_y),
								 c(gradient_threshold,
										  line_distance_threshold,
										  line_spread,
										  min_line_length,
										  roi_translation_x,
										  roi_translation_y),
								 coordinate(camera_height,
										 camera_angle,
										 aperture_angle_x,
										 aperture_angle_y,
										 cv::Size2i(width, height),
										 camera_translation),
								 coord(800, 0.4, 100),
								 ego_lane(point_amount),
								 frame(100),
								 screen_exists(std::stoi(argv[17]))
								 {}

		void loop(cv::Mat3b* src,
				  std::array<uint16_t, 10>* x_out,
				  std::array<int16_t, 10>* y_out,
				  std::array<int16_t, 10>* theta_out,
				  std::array<int16_t, 10>* kappa_out);


};

#endif /* SRC_MODULES_MAIN_LOOP_MAINLOOP_HPP_ */
